@@ -115,6 +115,11 @@ set( 'DOCHAZKA_DBPORT', '' );
 #set( 'DOCHAZKA_DBSSLMODE', 'require' );
 set( 'DOCHAZKA_DBSSLMODE', '' );
 
+# DOCHAZKA_AUDITING
+#    enable/disable auditing - note that if this is disabled at the beginning
+#    when the database is initialized, there is no easy way to enable it later
+set( 'DOCHAZKA_AUDITING', 1 );
+
 # DOCHAZKA_AUDIT_TABLES
 #    list of tables to audit (to disable auditing, set this parameter to [] in
 #    your SiteConfig.pm and call 'delete_audit_triggers')
