@@ -71,6 +71,17 @@ set( 'DBINIT_CREATE', [
 
     q#COMMENT ON FUNCTION round_time(timestamptz) IS 'Round a single timestamp value to the nearest 5 minutes'#,
 
+    q/CREATE OR REPLACE FUNCTION not_before_1892(timestamptz) 
+      RETURNS TIMESTAMPTZ AS $IMM$
+      BEGIN
+          IF $1 < '1892-01-01'::timestamptz THEN
+              RAISE EXCEPTION 'No dates earlier than 1892-01-01 please'; 
+          END IF;
+          RETURN $1;
+      END;
+    $IMM$ LANGUAGE plpgsql/,
+
+    q/COMMENT ON FUNCTION not_before_1892(timestamptz) IS 'We enforce dates 1892-01-01 or later'/,
 
     q#CREATE OR REPLACE FUNCTION valid_intvl() RETURNS trigger AS $$
         BEGIN
@@ -86,6 +97,8 @@ set( 'DBINIT_CREATE', [
                ( upper_inf(NEW.intvl) ) THEN
                 RAISE EXCEPTION 'illegal interval';
             END IF;
+            PERFORM not_before_1892(upper(NEW.intvl));
+            PERFORM not_before_1892(lower(NEW.intvl));
             IF ( upper(NEW.intvl) != round_time(upper(NEW.intvl)) ) OR
                ( lower(NEW.intvl) != round_time(lower(NEW.intvl)) ) THEN
                 RAISE EXCEPTION 'upper and lower bounds of interval must be evenly divisible by 5 minutes';
@@ -233,8 +246,9 @@ $body$#,
 
     q#CREATE TABLE IF NOT EXISTS schedules (
         sid        serial PRIMARY KEY,
+        scode      varchar(32) UNIQUE,
         schedule   text UNIQUE NOT NULL,
-        disabled   boolean,
+        disabled   boolean NOT NULL,
         remark     text,
         stamp      json
     )
@@ -257,6 +271,20 @@ $body$#,
     CREATE TRIGGER no_schedule_update BEFORE UPDATE ON schedules
       FOR EACH ROW EXECUTE PROCEDURE schedule_immutable()/,
     
+    q/-- trigger function to convert NULL to 'f' in boolean field
+    CREATE OR REPLACE FUNCTION disabled_to_zero() RETURNS trigger AS $$
+        BEGIN
+            IF NEW.disabled IS NULL THEN
+                NEW.disabled = 'f';
+            END IF;
+            RETURN NEW;
+        END;
+    $$ LANGUAGE plpgsql IMMUTABLE/,
+
+    q/-- trigger the disabled_to_zero trigger as well
+    CREATE TRIGGER disabled_to_zero BEFORE INSERT OR UPDATE ON schedules
+        FOR EACH ROW EXECUTE PROCEDURE disabled_to_zero()/,
+
     q/CREATE TABLE IF NOT EXISTS schedhistory (
         shid       serial PRIMARY KEY,
         eid        integer REFERENCES employees (eid) NOT NULL,
@@ -314,12 +342,25 @@ $body$#,
         END;
     $$ LANGUAGE plpgsql IMMUTABLE/,
 
+    q/CREATE OR REPLACE FUNCTION sane_timestamp() RETURNS trigger AS $$
+        BEGIN
+            PERFORM not_before_1892(NEW.effective);
+            RETURN NEW;
+        END;
+    $$ LANGUAGE plpgsql IMMUTABLE/,
+
     q/CREATE TRIGGER round_effective BEFORE INSERT OR UPDATE ON schedhistory
         FOR EACH ROW EXECUTE PROCEDURE round_effective()/,
 
     q/CREATE TRIGGER round_effective BEFORE INSERT OR UPDATE ON privhistory
         FOR EACH ROW EXECUTE PROCEDURE round_effective()/,
 
+    q/CREATE TRIGGER enforce_ts_sanity BEFORE INSERT OR UPDATE ON schedhistory
+        FOR EACH ROW EXECUTE PROCEDURE sane_timestamp()/,
+
+    q/CREATE TRIGGER enforce_ts_sanity BEFORE INSERT OR UPDATE ON privhistory
+        FOR EACH ROW EXECUTE PROCEDURE sane_timestamp()/,
+
     q#-- generalized function to get privilege level for an employee
       -- as of a given timestamp
       -- the complicated SELECT is necessary to ensure that the function
@@ -340,11 +381,11 @@ $body$#,
 
     q#-- this function will return 'jsonb' after PostgreSQL 9.4 comes out!!!
       CREATE OR REPLACE FUNCTION schedule_at_timestamp (INTEGER, TIMESTAMP WITH TIME ZONE)
-      RETURNS text AS $$
-          SELECT schedule FROM (
-              SELECT NULL AS schedule, '4713-01-01 BC' AS effective
+      RETURNS integer AS $$
+          SELECT sid FROM (
+              SELECT NULL AS sid, '4713-01-01 BC' AS effective
               UNION
-              SELECT schedules.schedule, schedhistory.effective
+              SELECT schedules.sid, schedhistory.effective
                   FROM schedules, schedhistory
                   WHERE schedules.sid = schedhistory.sid AND
                         schedhistory.eid=$1 AND 
@@ -363,7 +404,7 @@ $body$#,
     q#-- wrapper function to get schedule as of current timestamp
       -- this function will return 'jsonb' after PostgreSQL 9.4 comes out!!!
       CREATE OR REPLACE FUNCTION current_schedule (INTEGER)
-      RETURNS text AS $$
+      RETURNS integer AS $$
           SELECT schedule_at_timestamp($1, current_timestamp)
       $$ LANGUAGE sql IMMUTABLE#,
 
@@ -373,7 +414,7 @@ $body$#,
           code       varchar(32) UNIQUE NOT NULL,
           long_desc  text,
           remark     text,
-          disabled   boolean,
+          disabled   boolean NOT NULL,
           stamp      json,
           CONSTRAINT kosher_code CHECK (code ~* '^[[:alnum:]_][[:alnum:]_-]+$')
       )/,
@@ -402,6 +443,9 @@ $body$#,
     q/CREATE TRIGGER code_to_upper BEFORE INSERT OR UPDATE ON activities
         FOR EACH ROW EXECUTE PROCEDURE code_to_upper()/,
 
+    q/CREATE TRIGGER disabled_to_zero BEFORE INSERT OR UPDATE ON activities
+        FOR EACH ROW EXECUTE PROCEDURE disabled_to_zero()/,
+
     q/-- intervals
       CREATE TABLE intervals (
           iid        serial PRIMARY KEY,
@@ -601,7 +645,7 @@ $body$#,
         RETURNING eid
       ) 
       INSERT INTO privhistory (eid, priv, effective, remark)
-      SELECT eid, 'admin', '1000-01-01', 'IMMUTABLE' FROM cte
+      SELECT eid, 'admin', '1892-01-01', 'IMMUTABLE' FROM cte
     /,
 
     q/-- insert demo employee into employees table
@@ -632,6 +676,9 @@ set('DBINIT_MAKE_ROOT_IMMUTABLE', [
     CREATE OR REPLACE FUNCTION root_immutable() RETURNS trigger AS $IMM$
       BEGIN
           IF OLD.eid = ? THEN
+              IF NEW.eid <> ? THEN
+                  RAISE EXCEPTION 'root employee is immutable'; 
+              END IF;
               IF NEW.nick <> 'root' THEN
                   RAISE EXCEPTION 'root employee is immutable'; 
               END IF;
@@ -640,11 +687,6 @@ set('DBINIT_MAKE_ROOT_IMMUTABLE', [
       END;
     $IMM$ LANGUAGE plpgsql/,
     
-    q/
-    -- this trigger makes it impossible to update the root employee
-    CREATE TRIGGER no_root_change BEFORE UPDATE ON employees
-      FOR EACH ROW EXECUTE PROCEDURE root_immutable()/,
-    
     q/
     CREATE OR REPLACE FUNCTION root_immutable_new() RETURNS trigger AS $IMM$
       BEGIN
@@ -656,7 +698,19 @@ set('DBINIT_MAKE_ROOT_IMMUTABLE', [
     $IMM$ LANGUAGE plpgsql/,
     
     q/
-    CREATE OR REPLACE FUNCTION root_immutable_old() RETURNS trigger AS $IMM$
+    -- for use in BEFORE UPDATE triggers
+    CREATE OR REPLACE FUNCTION root_immutable_old_update() RETURNS trigger AS $IMM$
+      BEGIN
+          IF OLD.eid = ? THEN
+              RAISE EXCEPTION 'root employee is immutable'; 
+          END IF;
+          RETURN NEW;
+      END;
+    $IMM$ LANGUAGE plpgsql/,
+
+    q/
+    -- for use in BEFORE DELETE triggers
+    CREATE OR REPLACE FUNCTION root_immutable_old_delete() RETURNS trigger AS $IMM$
       BEGIN
           IF OLD.eid = ? THEN
               RAISE EXCEPTION 'root employee is immutable'; 
@@ -666,21 +720,32 @@ set('DBINIT_MAKE_ROOT_IMMUTABLE', [
     $IMM$ LANGUAGE plpgsql/,
 
     q/
-    -- this trigger makes it impossible to delete the root employee
-    CREATE TRIGGER no_root_delete BEFORE DELETE ON employees
-      FOR EACH ROW EXECUTE PROCEDURE root_immutable_old()/,
+    -- this trigger makes it impossible to update the root employee
+    CREATE TRIGGER no_root_change BEFORE UPDATE ON employees
+      FOR EACH ROW EXECUTE PROCEDURE root_immutable()/,
     
     q/
-    -- this trigger makes it impossible to insert any new privhistory 
+    -- this trigger makes it impossible to delete the root employee
+    CREATE TRIGGER no_root_delete BEFORE DELETE ON employees
+      FOR EACH ROW EXECUTE PROCEDURE root_immutable_old_delete()/,
+    
+    q/
+    -- this trigger makes it impossible to introduce any new privhistory 
     -- rows for the root employee
-    CREATE TRIGGER no_root_new BEFORE INSERT ON privhistory
+    CREATE TRIGGER no_root_new BEFORE INSERT OR UPDATE ON privhistory
       FOR EACH ROW EXECUTE PROCEDURE root_immutable_new()/,
     
     q/
-    -- this trigger makes it impossible to update or delete the root
+    -- this trigger makes it impossible to update the root
     -- employee's privhistory row
-    CREATE TRIGGER no_root_old BEFORE UPDATE OR DELETE ON privhistory
-      FOR EACH ROW EXECUTE PROCEDURE root_immutable_old()/,
+    CREATE TRIGGER no_root_update BEFORE UPDATE ON privhistory
+      FOR EACH ROW EXECUTE PROCEDURE root_immutable_old_update()/,
+    
+    q/
+    -- this trigger makes it impossible to delete the root
+    -- employee's privhistory row
+    CREATE TRIGGER no_root_old_delete BEFORE DELETE ON privhistory
+      FOR EACH ROW EXECUTE PROCEDURE root_immutable_old_delete()/,
     
 ]);
 
