@@ -43,6 +43,11 @@ set( 'SQL_EMPLOYEE_SELECT_BY_EID', q/
       SELECT eid, sec_id, nick, fullname, email, passhash, salt, remark
       FROM employees WHERE eid=?/ );
 
+# 
+set( 'SQL_EMPLOYEE_SELECT_BY_SEC_ID', q/
+      SELECT eid, sec_id, nick, fullname, email, passhash, salt, remark
+      FROM employees WHERE sec_id=?/ );
+
 #
 set( 'SQL_EMPLOYEE_SELECT_BY_NICK', q/
       SELECT eid, sec_id, nick, fullname, email, passhash, salt, remark
