@@ -47,6 +47,16 @@ set( 'SQL_PRIVHISTORY_INSERT', q/
       RETURNING phid, eid, priv, effective, remark
       / );
 
+# SQL_PRIVHISTORY_UPDATE
+#     SQL to update a single row from privhistory table
+#
+set( 'SQL_PRIVHISTORY_UPDATE', q/
+      UPDATE privhistory 
+      SET priv = ?, effective = ?, remark = ?
+      WHERE phid = ?
+      RETURNING phid, eid, priv, effective, remark
+      / );
+
 # SQL_PRIVHISTORY_DELETE
 #     SQL to delete a single row from privhistory table
 #
