@@ -58,31 +58,33 @@ App::Dochazka::REST - Dochazka REST server
 
 =head1 VERSION
 
-Version 0.407
+Version 0.427
 
 =cut
 
-our $VERSION = '0.407';
+our $VERSION = '0.427';
 
 
 =head2 Development status
 
-Dochazka is currently a Work In Progress (WIP). Do not expect it to do
-anything useful.
-
-The REST server component of Dochazka is nearing completion.
+Alpha.
 
 
 
 
 =head1 SYNOPSIS
 
-This module, the top-level module of the Dochazka REST server, contains
-routines for (re-)initializing the database and initializing the server (mainly
-loading configuration parameters and starting the logger). 
+Start the server with default settings:
 
-The module also contains documentation of the Dochazka REST server as a whole.
+    $ dochazka-rest
 
+Point browser to:
+
+    http://localhost:5000/
+
+Use L<App::Dochazka::CLI> command-line interface to access full functionality:
+
+    $ dochazka-cli
 
 
 =head1 DESCRIPTION
@@ -125,10 +127,11 @@ to divine their intent and express it in terms of HTTP requests to the REST
 server.
 
 The HTTP protocol is used in all communication between client and server. In
-Dochazka, the term "client" should be understood in a broad sense, encompassing
-stand-alone report generators, specialized administration utilities, cronjobs,
-web browsers, etc., in addition to the purpose-built Perl (CLI) and JavaScript
-(WWW) clients. 
+Dochazka, the term "client" should be understood in a broad sense to mean
+anything that communicates with the server using the HTTP protocol. This
+encompasses stand-alone report generators, specialized administration
+utilities, cronjobs, web browsers, etc., in addition to the purpose-built
+clients or just plain C<curl>.
 
 =back
 
@@ -136,10 +139,10 @@ web browsers, etc., in addition to the purpose-built Perl (CLI) and JavaScript
 =head2 Client-server communication
 
 As stated above, communication between the server and its clients takes place
-using the HTTP protocol. At an even more fundamental level, the communication
-takes the form of requests (from client to server) and responses (from server
-back to client) to those requests. In other words, communication is never
-initiated by the server, but always by the clients.
+using the HTTP protocol. More abstractly, the communication takes the form of
+requests (from client to server) and responses (from server back to client) to
+those requests. In other words, communication is never initiated by the server,
+but always by the clients.
 
 =head3 HTTP request
 
@@ -147,20 +150,29 @@ An HTTP request has the following basic components:
 
 =over
 
-=item * Method (GET, PUT, POST, or DELETE)
+=item * Method 
 
-=item * URI (Universal Resource Indicator)
+Dochazka supports GET, PUT, POST, and DELETE
+
+=item * URI 
+
+Universal Resource Indicator specifying a Dochazka resource
 
 =item * Headers
 
-=item * Content body (or "request entity" - PUT, POST, DELETE only)
+More on these below
+
+=item * Request entity
+
+Data accompanying the request - may or may not be present
 
 =back
 
+
 =head4 Method
 
-The Dochazka REST server accepts the following HTTP methods:  C<PUT>,
-C<POST>, and C<DELETE>.
+The Dochazka REST server accepts the following HTTP methods:  
+C<GET>, C<PUT>, C<POST>, and C<DELETE>.
 
 =over
 
@@ -168,39 +180,36 @@ C<POST>, and C<DELETE>.
 
 A C<GET> request on a resource is a request for information - in other words,
 it is "read-only": C<GET> requests never change the underlying data. In
-SQL terms, C<GET> requests are typically mapped to C<SELECT> statements.
+Dochazka, C<GET> requests frequently map to C<SELECT> statements.
 
 =item C<PUT>
 
 C<PUT> requests always refer to a concrete data entity, or chunk of data.
-Typically this will be a record in the underlying database. If the record
-already exists, the C<PUT> request is interpreted to mean modification (or
-C<UPDATE> in SQL). If the record does not exist, then the request will map to
-an C<INSERT> statement to create the resource. If the request causes a new
-resource to be created, the location of the new resource should be returned in
-the response headers.
+In simple cases, this will be a single record in the underlying database. If
+the record already exists, the C<PUT> request is interpreted to mean
+modification (or C<UPDATE> in SQL). If the record does not exist, then the
+request will map to an C<INSERT> statement to create the resource. In both
+cases, upon success the response status will be C<200 OK>.
 
 =item C<POST>
 
 Sometimes, especially for create operations, the exact specification of the
 resource is not known beforehand. To address these cases, some resources accept
-C<POST> requests. Like with C<PUT>, links to newly created resources are
-supposed to be returned in the response headers.
+C<POST> requests. If the request causes a new resource to be created, the HTTP
+response status will be C<201 Created> and there will be a C<Location> header
+specifying the URI of the newly created resource.
 
 =item C<DELETE>
 
 As their name would suggest, C<DELETE> requests are issued when we want to
-dissolve or destroy a resource. Whether or not this actually happens is
+dissolve (destroy) a resource. Whether or not this actually happens is
 determined by two factors: (1) whether the user issuing the request has the
 requisite authorization and, (2), whether the underlying data record is
 referred to by other records - in which case typically the C<DELETE> request
-will fail.
+will fail with a C<500 Internal Server Error> status.
 
 =back
 
-Of these four methods, only two (C<GET> and C<POST>) really get used much on
-the World Wide Web. Still, they are part of the HTTP standard and they are
-useful for our purposes.
 
 
 =head4 URI
@@ -208,9 +217,6 @@ useful for our purposes.
 The purpose of the Universal Resource Indicator (URI, sometimes also known as
 an URL) is to uniquely identify a resource.
 
-While Dochazka only accepts four HTTP methods, the number of URIs it will
-accept is much higher.
-
 URIs consist of several syntactical elements. An exhaustive description can be
 found in RFC ..., but for Dochazka purposes we can present them as follows:
 
@@ -222,34 +228,43 @@ This part of the URI says that we are using the HTTPS protocol (or SSL-encrypted
 HTTP) to communicate. It is separated from the next component by two forward
 slashes.
 
-=item C<dochazka.site/>
+=item C<dochazka.site>
 
 After the protocol, the next URI component is the REST server's domain name.
 Obviously, this will differ from site to site. It is separated from the next
-component by a single forward slash.
+component (i.e. the resource specification) by a single forward slash.
 
 =item Dochazka resource
 
 As stated above, the domain name is terminated by a single forward slash.
 Everything after that is interpreted as a resource specification. 
 
+A single forward slash C<'/'> specifies the root resource.
+
 =back
 
-Of these three components, the first two can be abstracted away as site-specific. 
+Of these three components, the first two are site-specific. It is possible, 
+for example, to run the Dochazka server without SSL encryption, in which case
+the protocol would be C<http://> instead of C<https://>.
+
 Once the application's implementation at a given site has stabilized, these two
 URI components will change very seldomly, if at all.
 
-Dochazka resources, on the other hand, are much more ephemeral. Different resources
-present different ways that users can access and modify the data (in this case,
-attendance data) in the underlying database. Some resources, such as
-C<employee/nick/simona>, refer directly to a unit of information that may or
-may not exist in the database information. Other resources, like
-C<interval/new>, are not linked to a specific database record.
+Dochazka resources are much more ephemeral. Different resources present
+different ways that users can access and modify the data (in this case,
+attendance data) in the underlying database. 
 
-(Regardless of how we think of them, internally all resources resolve to
-function calls. The function in this case is referred to as the "dispatch
-target" and the argument provided in the function call is the "request
-context".)
+Some resources, such as C<employee/nick/simona>, refer directly to a unit of
+information that may or may not exist in the database information. Other
+resources, like C<interval/new>, are not linked to a specific database record.
+
+Also, in programming terms the resources are generalized, so we think about,
+e.g., C<employee/nick/simona> and C<employee/nick/wanda> as two instances of a
+more generalized C<employee/nick/:nick> resource, where C<:nick> is like an
+argument to a function call.
+
+And, indeed, internally all resources resolve to function calls. The function
+in this case is referred to as the "resource handler".
 
 Some resources accept all four HTTP methods listed above, others accept two or
 three, and still others accept only one.
@@ -1461,8 +1476,7 @@ sub reset_db {
     };
     return $status unless $status->ok;
     
-    # create:
-    # - audit triggers on all the tables
+    # if auditing is enabled, create the audit triggers
     if ( $site->DOCHAZKA_AUDITING ) {
         $status = create_audit_triggers( $conn );
         return $status unless $status->ok;
