@@ -55,11 +55,11 @@ App::Dochazka::REST::Dispatch::ACL - ACL module
 
 =head1 VERSION
 
-Version 0.407
+Version 0.427
 
 =cut
 
-our $VERSION = '0.407';
+our $VERSION = '0.427';
 
 
 
@@ -188,9 +188,11 @@ sub acl_check_is_me {
 
     if ( my $eid = $pl{'eid'} ) {
         $log->debug( "acl_check_is_me: I am EID " . $ce->eid . " - checking against $eid" );
-        return ( $eid == $ce->eid );
+        return ( defined($eid) and $eid == $ce->eid );
     } elsif ( my $nick = $pl{'nick'} ) {
-        return ( $nick eq $ce->nick );
+        return ( defined($nick) and $nick eq $ce->nick );
+    } elsif ( my $sec_id = $pl{'sec_id'} ) {
+        return ( defined($sec_id) and $sec_id eq $ce->sec_id );
     }
 
     die "AAAAGAGAGAHHHHAHAHAAJJAJAJAJAAHAHAHA! " . Dumper( \%pl );
