@@ -53,7 +53,7 @@ use App::Dochazka::REST::Model::Schedule qw( get_all_schedules );
 use App::Dochazka::REST::Model::Shared qw( load_multiple priv_by_eid schedule_by_eid );
 use App::Dochazka::REST::ResourceDefs;
 use App::Dochazka::REST::Shared qw( :ALL );  # all the shared_* functions
-use App::Dochazka::REST::Util qw( hash_the_password pre_update_comparison );
+#use App::Dochazka::REST::Util qw( hash_the_password pre_update_comparison );
 use App::Dochazka::REST::Util::Timestamp qw( $current_year $current_month );
 use Data::Dumper;
 use Module::Runtime qw( use_module );
@@ -77,11 +77,11 @@ App::Dochazka::REST::Dispatch - Implementation of top-level resources
 
 =head1 VERSION
 
-Version 0.407
+Version 0.427
 
 =cut
 
-our $VERSION = '0.407';
+our $VERSION = '0.427';
 
 
 =head1 DESCRIPTION
@@ -1000,31 +1000,7 @@ Handler for 'GET employee/eid/:eid'
 sub handler_get_employee_eid {
     my ( $self, $pass ) = @_;
     $log->debug( "Entering " . __PACKAGE__ . "::handler_get_employee_eid" ); 
-
-    # first pass
-    if ( $pass == 1 ) {
-        #,
-        # 403 (ACL) check - passerby can only look up him- or herself
-        #
-        my $eid = $self->context->{'mapping'}->{'eid'};
-        if ( ! acl_check_is_me( $self, 'eid' => $eid ) ) {
-            $self->mrest_declare_status( code => 403, explanation => "DISPATCH_KEEP_TO_YOURSELF" );
-            return 0;
-        }
-        # 
-        # 404 check
-        #
-        my $emp = shared_first_pass_lookup( $self, 'EID', $eid );
-        return 0 unless $emp;
-        #return 0 unless shared_employee_acl_part1( $self, $emp );  # additional ACL checks
-        $self->context->{'stashed_employee_object'} = $emp;
-        return 1;
-    }
-
-    # second pass
-    return $CELL->status_ok( 'DISPATCH_EMPLOYEE_FOUND',
-        payload => $self->context->{'stashed_employee_object'},
-    );
+    return shared_get_employee( $self, $pass, 'EID', $self->context->{'mapping'}->{'eid'} );
 }
 
 
@@ -1057,31 +1033,23 @@ Handler for 'GET employee/nick/:nick'
 sub handler_get_employee_nick {
     my ( $self, $pass ) = @_;
     $log->debug( "Entering " . __PACKAGE__ . "::handler_get_employee_nick" ); 
-
-    # first pass
-    if ( $pass == 1 ) {
-        #
-        # ACL check - passerby can only look up him- or herself
-        my $priv = $self->context->{'current_priv'};
-        my $nick = $self->context->{'mapping'}->{'nick'};
-        if ( $priv ne 'admin' ) {
-            if ( ! acl_check_is_me( $self, 'nick' => $nick ) ) {
-                $self->mrest_declare_status( code => 403, explanation => "DISPATCH_KEEP_TO_YOURSELF" );
-                return 0;
-            }
-        }
-        my $emp = shared_first_pass_lookup( $self, 'nick', $self->context->{'mapping'}->{'nick'} );
-        return 0 unless $emp;
-        $self->context->{'stashed_employee_object'} = $emp;
-        return 1;
-    }
-
-    # second pass
-    return $CELL->status_ok( 'DISPATCH_EMPLOYEE_FOUND',
-        payload => $self->context->{'stashed_employee_object'},
-    );
+    return shared_get_employee( $self, $pass, 'nick', $self->context->{'mapping'}->{'nick'} );
 }
 
+
+=head3 handler_get_employee_sec_id
+
+Handler for 'GET employee/sec_id/:sec_id'
+
+=cut
+
+sub handler_get_employee_sec_id {
+    my ( $self, $pass ) = @_;
+    $log->debug( "Entering " . __PACKAGE__ . "::handler_get_employee_sec_id" ); 
+    return shared_get_employee( $self, $pass, 'sec_id', $self->context->{'mapping'}->{'sec_id'} );
+}
+
+
 =head3 handler_get_employee_search_nick
 
 Handler for 'GET employee/search/nick/:key'
@@ -1280,6 +1248,33 @@ sub handler_history_get_phid {
     );
 }
 
+=head3 handler_history_post_phid
+
+Handler for 'POST priv/history/phid/:phid'
+
+=cut
+
+sub handler_history_post_phid {
+    my ( $self, $pass ) = @_;
+    $log->debug( "Entering " . __PACKAGE__ . "::handler_history_post_phid" ); 
+
+    # first pass
+    if ( $pass == 1 ) {
+        my $p_obj = shared_first_pass_lookup( $self, 'PHID', $self->context->{'mapping'}->{'phid'} );
+        return 0 unless $p_obj;
+        $self->context->{'stashed_history_object'} = $p_obj;
+        return 1;
+    }
+
+    # second pass
+    return shared_update_history( 
+        $self,
+        $self->context->{'stashed_history_object'}, 
+        $self->context->{'request_entity'} 
+    );
+}
+
+
 =head3 handler_history_delete_phid
 
 =cut
@@ -1317,6 +1312,32 @@ sub handler_history_get_shid {
     );
 }
 
+=head3 handler_history_post_shid
+
+Handler for 'POST priv/history/shid/:shid'
+
+=cut
+
+sub handler_history_post_shid {
+    my ( $self, $pass ) = @_;
+    $log->debug( "Entering " . __PACKAGE__ . "::handler_history_post_shid" ); 
+
+    # first pass
+    if ( $pass == 1 ) {
+        my $p_obj = shared_first_pass_lookup( $self, 'SHID', $self->context->{'mapping'}->{'shid'} );
+        return 0 unless $p_obj;
+        $self->context->{'stashed_history_object'} = $p_obj;
+        return 1;
+    }
+
+    # second pass
+    return shared_update_history( 
+        $self,
+        $self->context->{'stashed_history_object'}, 
+        $self->context->{'request_entity'} 
+    );
+}
+
 =head3 handler_history_delete_shid
 
 =cut
@@ -1914,8 +1935,8 @@ sub handler_schedule_all {
         my $status = get_all_schedules( conn => $self->context->{'dbix_conn'}, disabled => $disabled );
         if ( $status->level eq 'NOTICE' and $status->code eq 'DISPATCH_NO_RECORDS_FOUND' ) {
             my $explanation = ( $disabled )
-                ? 'There are no schedules in the database'
-                : 'There are no active schedules in the database';
+                ? 'DISPATCH_NO_SCHEDULES'
+                : 'DISPATCH_NO_ACTIVE_SCHEDULES';
             $self->mrest_declare_status( explanation => $explanation );
             return 0;
         }
@@ -2019,28 +2040,35 @@ sub handler_schedule_new {
 
     #
     # spawn Schedule object
-    my $sched = App::Dochazka::REST::Model::Schedule->spawn(
-        'schedule' => $intvls->json,
-    );
+    my @ARGS = ( 'schedule' => $intvls->json );
+    if ( my $scode = $context->{'request_entity'}->{'scode'} ) {
+        push @ARGS, ( 'scode' => $scode );
+    }
+    my $sched = App::Dochazka::REST::Model::Schedule->spawn( @ARGS );
     #
     # insert schedule object to get SID
     $status = $sched->insert( $context );
     if ( $status->ok ) {
         if ( $status->code eq 'DOCHAZKA_SCHEDULE_EXISTS' ) {
             $self->context->{'create_path'} = '.../schedule/shid/' . $sched->sid;
-            $code = 'DISPATCH_SCHEDULE_OK';
-            $log->info( "schedule/intervals: Found existing schedule" );
-        } elsif ( $status->code eq 'DOCHAZKA_CUD_OK' ) {
+            $code = 'DISPATCH_SCHEDULE_EXISTS';
+            $log->info( "POST schedule/new: Returning existing schedule, unchanged" );
+            $sched = $status->payload;
+        } elsif ( $status->code eq 'DOCHAZKA_SCHEDULE_UPDATE_OK' ) {
+            $self->context->{'create_path'} = '.../schedule/shid/' . $sched->sid;
+            $code = 'DISPATCH_SCHEDULE_UPDATE_OK';
+            $log->info( "POST schedule/new: Existing schedule updated" );
+        } elsif ( $status->code eq 'DOCHAZKA_SCHEDULE_INSERT_OK' ) {
             $self->context->{'create_path'} = '.../schedule/shid/' . $sched->sid;
             $code = 'DISPATCH_SCHEDULE_INSERT_OK';
-            $log->info( "schedule/intervals: New schedule inserted" );
+            $log->info( "POST schedule/new: New schedule inserted" );
         } else {
-            $log->crit( "schedule/intervals: Unknown status code returned by Model/Schedule.pm->insert" );
-            die( 'AAAAAAAAHHHHH! Swallowed by the abyss' );
+            die "AGGHGHG! could not handle App::Dochazka::REST::Model::Schedule->insert status: " 
+                . Dumper( $status );
         }
     } else {
         $self->mrest_declare_status( code => 500, explanation => 
-            "schedule/intervals: Model/Schedule.pm->insert failed - bailing out" );
+            "schedule/intervals: Model/Schedule.pm->insert failed: " . $status->text );
         $intvls->delete( $context->{'dbix_conn'} );
         return $fail;
     }
@@ -2135,4 +2163,82 @@ sub handler_delete_schedule_sid {
     return $context->{'stashed_schedule_object'}->delete( $context );
 }
 
+
+=head3 handler_get_schedule_scode
+
+Handler for '/schedule/scode/:scode'
+
+=cut
+
+sub handler_get_schedule_scode {
+    my ( $self, $pass ) = @_;
+    $log->debug( "Entering " . __PACKAGE__ . ":handler_get_schedule_scode" ); 
+
+    # first pass
+    if ( $pass == 1 ) {
+        my $sched = shared_first_pass_lookup( $self, 'scode', $self->context->{'mapping'}->{'scode'} );
+        return 0 unless $sched;
+        $self->context->{'stashed_schedule_object'} = $sched;
+        return 1;
+    }
+    
+    # second pass
+    return $CELL->status_ok( 
+        'DISPATCH_SCHEDULE_FOUND',
+        payload => $self->context->{'stashed_schedule_object'},
+    );
+}
+
+
+=head3 handler_put_schedule_scode
+
+Handler for 'PUT schedule/scode/:scode'
+
+=cut
+
+sub handler_put_schedule_scode {
+    my ( $self, $pass ) = @_;
+    $log->debug( "Entering " . __PACKAGE__ . ":handler_put_schedule_scode" ); 
+
+    my $context = $self->context;
+    my $scode = $context->{'mapping'}->{'scode'};
+
+    # first pass
+    if ( $pass == 1 ) {
+         my $sched = shared_first_pass_lookup( $self, 'scode', $scode );
+         return 0 unless $sched;
+         $context->{'stashed_schedule_object'} = $sched;
+         return 1;
+    }
+
+    # run the update operation
+    return shared_update_schedule( 
+        $self,
+        $context->{'stashed_schedule_object'}, 
+        $context->{'request_entity'} 
+    );
+}
+
+
+=head3 handler_delete_schedule_scode
+
+Handler for '/schedule/scode/:scode'
+
+=cut
+
+sub handler_delete_schedule_scode {
+    my ( $self, $pass ) = @_;
+    $log->debug( "Entering " . __PACKAGE__ . ":handler_delete_schedule_scode" ); 
+
+    my $context = $self->context;
+
+    # first pass
+    if ( $pass == 1 ) {
+        return $self->handler_get_schedule_scode( $pass );
+    }
+
+    # second pass
+    return $context->{'stashed_schedule_object'}->delete( $context );
+}
+
 1;
