@@ -63,11 +63,11 @@ App::Dochazka::REST::Model::Employee - Employee data model
 
 =head1 VERSION
 
-Version 0.407
+Version 0.427
 
 =cut
 
-our $VERSION = '0.407';
+our $VERSION = '0.427';
 
 
 
@@ -417,6 +417,30 @@ sub load_by_nick {
 }
 
 
+=head2 load_by_sec_id
+
+Analogous method to L<App::Dochazka::REST::Model::Activity/"load_by_aid">.
+
+FIXME: add unit tests
+
+=cut
+
+sub load_by_sec_id {
+    my $self = shift;
+    my ( $conn, $sec_id ) = validate_pos( @_, 
+        { isa => 'DBIx::Connector' },
+        { type => SCALAR },
+    );
+    $log->debug( "Entering " . __PACKAGE__ . "::load_by_sec_id with argument $sec_id" );
+
+    return load( 
+        conn => $conn,
+        class => __PACKAGE__, 
+        sql => $site->SQL_EMPLOYEE_SELECT_BY_SEC_ID,
+        keys => [ $sec_id ], 
+    );
+}
+
 
 
 =head1 FUNCTIONS
