@@ -56,11 +56,11 @@ App::Dochazka::REST::Model::Privhistory - privilege history functions
 
 =head1 VERSION
 
-Version 0.407
+Version 0.427
 
 =cut
 
-our $VERSION = '0.407';
+our $VERSION = '0.427';
 
 
 
@@ -280,8 +280,24 @@ sub insert {
 
 =head2 update
 
-There is no 'update' method for privhistory records. Instead, delete and
-re-recreate.
+Instance method. Updates the record. Returns status object.
+
+=cut
+
+sub update {
+    my $self = shift;
+    my ( $context ) = validate_pos( @_, { type => HASHREF } );
+
+    my $status = cud(
+        conn => $context->{'dbix_conn'},
+        eid => $context->{'current'}->{'eid'},
+        object => $self,
+        sql => $site->SQL_PRIVHISTORY_UPDATE,
+        attrs => [ 'priv', 'effective', 'remark', 'phid' ],
+    );
+
+    return $status;
+}
 
 
 =head2 delete
