@@ -55,11 +55,11 @@ App::Dochazka::REST::Model::Schedhistory - schedule history functions
 
 =head1 VERSION
 
-Version 0.407
+Version 0.427
 
 =cut
 
-our $VERSION = '0.407';
+our $VERSION = '0.427';
 
 
 
@@ -250,8 +250,24 @@ sub insert {
 
 =head2 update
 
-There is no update method for schedhistory records. Instead, delete and
-re-create.
+Instance method. Updates the record. Returns status object.
+
+=cut
+
+sub update {
+    my $self = shift;
+    my ( $context ) = validate_pos( @_, { type => HASHREF } );
+
+    my $status = cud(
+        conn => $context->{'dbix_conn'},
+        eid => $context->{'current'}->{'eid'},
+        object => $self,
+        sql => $site->SQL_SCHEDHISTORY_UPDATE,
+        attrs => [ 'sid', 'effective', 'remark', 'shid' ],
+    );
+
+    return $status;
+}
 
 
 =head2 delete
