@@ -57,11 +57,11 @@ App::Dochazka::REST::Model::Schedule - schedule functions
 
 =head1 VERSION
 
-Version 0.407
+Version 0.427
 
 =cut
 
-our $VERSION = '0.407';
+our $VERSION = '0.427';
 
 
 
@@ -281,6 +281,11 @@ our @EXPORT_OK = qw(
 Instance method. Attempts to INSERT a record into the 'schedules' table.
 Field values are taken from the object. Returns a status object.
 
+If the "schedule" field of the schedule to be inserted matches an existing
+schedule, no new record is inserted. Instead, the existing schedule record
+is returned. In such a case, the "scode", "remark", and "disabled" fields
+are ignored - except when they are NULL in the existing record.
+
 =cut
 
 sub insert {
@@ -291,14 +296,50 @@ sub insert {
     # don't insert it again
     my $status = select_single( 
         conn => $context->{'dbix_conn'}, 
-        sql => $site->SQL_SCHEDULES_SELECT_SID, 
+        sql => $site->SQL_SCHEDULES_SELECT_BY_SCHEDULE, 
         keys => [ $self->{schedule} ],
     );
-    $log->debug( Dumper $status );
+    $log->info( "select_single returned: " . Dumper $status );
     if ( $status->level eq 'OK' ) {
-        ( $self->{'sid'} ) = @{ $status->payload };
-        return $CELL->status_ok( 'DOCHAZKA_SCHEDULE_EXISTS', args => [ $self->{sid} ] );
+        my $found_sched = App::Dochazka::REST::Model::Schedule->spawn( 
+            sid => $status->payload->[0],
+            scode => $status->payload->[1],
+            schedule => $status->payload->[2],
+            remark => $status->payload->[3],
+            disabled => $status->payload->[4],
+        );
+        $self->{'sid'} = $found_sched->sid;
+        {
+            #
+            # the exact schedule exists, but if any of { scode, remark, disabled }
+            # are NULL and we have a value, update the record to reflect the value
+            # (in other words, do not prefer NULLs over real values)
+            #
+            my $do_update = 0;
+            if ( ! defined( $found_sched->scode ) and defined( $self->scode ) ) {
+                $found_sched->scode( $self->scode );
+                $do_update = 1;
+            }
+            if ( ! defined( $found_sched->remark ) and defined( $self->remark ) ) {
+                $found_sched->remark( $self->remark );
+                $do_update = 1;
+            }
+            if ( ! defined( $found_sched->disabled ) and defined( $self->disabled ) ) {
+                $found_sched->disabled( $self->disabled );
+                $do_update = 1;
+            }
+            if ( $do_update ) {
+                $status = $found_sched->update( $context );
+                if ( $status->level eq 'OK' and $status->code eq 'DOCHAZKA_CUD_OK' ) {
+                    $status->code( 'DOCHAZKA_SCHEDULE_UPDATE_OK' );
+                }
+                return $status;
+            }
+            return $CELL->status_ok( 'DOCHAZKA_SCHEDULE_EXISTS', args => [ $self->{sid} ],
+                payload => $found_sched );
+        }
     } elsif( $status->level ne 'NOTICE' ) {
+        $log->info( "select_single status was neither OK nor NOTICE; returning it as-is" );
         return $status;
     }
 
@@ -309,11 +350,13 @@ sub insert {
         eid => $context->{'current'}->{'eid'},
         object => $self,
         sql => $site->SQL_SCHEDULE_INSERT,
-        attrs => [ 'schedule', 'remark' ],
+        attrs => [ 'scode', 'schedule', 'remark' ],
     );
 
-    $log->info( "Inserted new schedule with SID" . $self->{sid} ) if $status->ok;
-
+    if ( $status->ok ) {
+        $status->code( 'DOCHAZKA_SCHEDULE_INSERT_OK' );
+        $log->info( "Inserted new schedule with SID " . $self->{sid} );
+    }
     return $status;
 }
 
@@ -321,7 +364,7 @@ sub insert {
 =head2 update
 
 Although we do not allow the 'sid' or 'schedule' fields to be updated, schedule
-records have 'remark' and 'disabled' fields that can be updated via this
+records have 'scode', 'remark' and 'disabled' fields that can be updated via this
 method. 
 
 =cut
@@ -337,7 +380,7 @@ sub update {
         eid => $context->{'current'}->{'eid'},
         object => $self,
         sql => $site->SQL_SCHEDULE_UPDATE,
-        attrs => [ 'remark', 'disabled', 'sid' ],
+        attrs => [ 'scode', 'remark', 'disabled', 'sid' ],
     );
 
     return $status;
@@ -369,6 +412,29 @@ sub delete {
 }
 
 
+=head2 load_by_scode
+
+Analogous function to L<App::Dochazka::REST::Model::Activity/"load_by_aid">.
+
+=cut
+
+sub load_by_scode {
+    my $self = shift;
+    my ( $conn, $scode ) = validate_pos( @_,
+        { isa => 'DBIx::Connector' },
+        { type => SCALAR },
+    );
+
+    return load( 
+        conn => $conn,
+        class => __PACKAGE__, 
+        sql => $site->SQL_SCHEDULE_SELECT_BY_SCODE,
+        keys => [ $scode ],
+    );
+}
+
+
+
 =head2 load_by_sid
 
 Analogous function to L<App::Dochazka::REST::Model::Activity/"load_by_aid">.
