@@ -54,11 +54,11 @@ the data model
 
 =head1 VERSION
 
-Version 0.407
+Version 0.427
 
 =cut
 
-our $VERSION = '0.407';
+our $VERSION = '0.427';
 
 
 
@@ -472,8 +472,8 @@ sub priv_by_eid {
 
 =head2 schedule_by_eid
 
-Given an EID, and, optionally, a timestamp, returns the employee's schedule
-as of that timestamp, or as of "now" if no timestamp was given.
+Given an EID, and, optionally, a timestamp, returns the SID of the employee's
+schedule as of that timestamp, or as of "now" if no timestamp was given.
 
 =cut
 
@@ -529,8 +529,6 @@ sub _st_by_eid {
     };
     return $status if $status;
 
-    $row = decode_schedule_json( $row ) if $st eq 'schedule';
-
     $log->debug( "_st_by_eid success; returning payload " . Dumper( $row ) );
     return $row;
 }
@@ -558,7 +556,7 @@ sub select_single {
         $ARGS{'conn'}->run( fixup => sub {
             @results = $_->selectrow_array( $ARGS{'sql'}, undef, @{ $ARGS{'keys'} } );
         } );
-        my $count = scalar @results;
+        my $count = scalar( @results ) ? 1 : 0;
         $status = ( $count )
             ? $CELL->status_ok( 'DISPATCH_RECORDS_FOUND', 
                 args => [ $count ], count => $count, payload => \@results )
