@@ -54,11 +54,11 @@ App::Dochazka::REST::ResourceDefs - Resource definitions
 
 =head1 VERSION
 
-Version 0.407
+Version 0.427
 
 =cut
 
-our $VERSION = '0.407';
+our $VERSION = '0.427';
 
 
 
@@ -698,6 +698,7 @@ EOH
     # /employee/eid/:eid
     'employee/eid/:eid' =>
     { 
+        parent => 'employee',
         handler => {
             GET => 'handler_get_employee_eid', 
             PUT => 'handler_put_employee_eid', 
@@ -796,7 +797,7 @@ EOH
         },
         cli => 'employee nick $NICK [$JSON]',
         validations => {
-            'nick' => qr/^[%[:alnum:]_][%[:alnum:]_-]+$/,
+            'nick' => qr/^[%[:alnum:]_][%[:alnum:]_-]*$/,
         },
         description => "Retrieves (GET), updates/inserts (PUT), and/or deletes (DELETE) the employee specified by the ':nick' parameter",
         documentation => <<'EOH',
@@ -890,7 +891,7 @@ EOH
         cli => 'employee sec_id $SEC_ID',
         description => 'GET an employee profile by the employee\'s secondary ID',
         validations => {
-            'sec_id' => qr/^[%[:alnum:]_][%[:alnum:]_-]+$/,
+            'sec_id' => qr/^[%[:alnum:]_][%[:alnum:]_-]*$/,
         },
         documentation => <<'EOH',
 =pod
@@ -939,6 +940,7 @@ EOH
     # /employee/self/priv
     'employee/self/priv' =>
     { 
+        parent => 'employee/self',
         handler => {
             GET => 'handler_get_employee_self_privsched',
         },
@@ -997,7 +999,7 @@ EOH
     {
         parent => 'schedule',
         handler => 'handler_noop',
-        cli => 'sched history',
+        cli => 'schedule history',
         description => 'Schedule history resources',
         documentation => <<'EOH',
 =pod
@@ -1013,7 +1015,10 @@ EOH
             GET => 'handler_history_get', 
             POST => 'handler_history_post',
         },
-        acl_profile => 'admin',
+        acl_profile => {
+            GET => 'inactive',
+            POST => 'admin',
+        },
         cli => 'priv history eid $EID [$JSON]',
         validations => {
             'eid' => 'Int',
@@ -1058,8 +1063,11 @@ EOH
             GET => 'handler_history_get', 
             POST => 'handler_history_post',
         },
-        acl_profile => 'admin',
-        cli => 'sched history eid $EID [$JSON]',
+        acl_profile => {
+            GET => 'inactive',
+            POST => 'admin',
+        },
+        cli => 'schedule history eid $EID [$JSON]',
         validations => {
             'eid' => 'Int',
         },
@@ -1101,7 +1109,7 @@ EOH
         handler => {
             GET => 'handler_history_get', 
         },
-        acl_profile => 'admin',
+        acl_profile => 'inactive',
         cli => 'priv history eid $EID $TSRANGE',
         description => 'Get a slice of history of privilege level changes for employee with the given EID',
         validations => {
@@ -1122,8 +1130,8 @@ EOH
         handler => {
             GET => 'handler_history_get', 
         },
-        acl_profile => 'admin',
-        cli => 'sched history eid $EID $TSRANGE',
+        acl_profile => 'inactive',
+        cli => 'schedule history eid $EID $TSRANGE',
         description => 'Get a slice of history of schedule changes for employee with the given EID',
         validations => {
             'eid' => 'Int',
@@ -1144,10 +1152,13 @@ EOH
             GET => 'handler_history_get', 
             POST => 'handler_history_post', 
         },
-        acl_profile => 'admin',
+        acl_profile => {
+            GET => 'inactive',
+            POST => 'admin',
+        },
         cli => 'priv history nick $NICK [$JSON]',
         validations => {
-            'nick' => qr/^[[:alnum:]_][[:alnum:]_-]+$/,
+            'nick' => qr/^[[:alnum:]_][[:alnum:]_-]*$/,
         },
         description => 'Retrieves entire history of privilege level changes for employee with the given nick (GET); or, with an appropriate content body, adds (PUT) a record to employee\'s privhistory',
         documentation => <<'EOH',
@@ -1189,10 +1200,13 @@ EOH
             GET => 'handler_history_get', 
             POST => 'handler_history_post', 
         },
-        acl_profile => 'admin',
-        cli => 'sched history nick $NICK [$JSON]',
+        acl_profile => {
+            GET => 'inactive',
+            POST => 'admin',
+        },
+        cli => 'schedule history nick $NICK [$JSON]',
         validations => {
-            'nick' => qr/^[[:alnum:]_][[:alnum:]_-]+$/,
+            'nick' => qr/^[[:alnum:]_][[:alnum:]_-]*$/,
         },
         description => 'Retrieves entire history of schedule changes for employee with the given nick (GET); or, with an appropriate content body, adds (PUT) a record to employee\'s schedule history',
         documentation => <<'EOH',
@@ -1232,10 +1246,10 @@ EOH
         handler => {
             GET => 'handler_history_get', 
         },
-        acl_profile => 'admin',
+        acl_profile => 'inactive',
         cli => 'priv history nick $NICK $TSRANGE',
         validations => {
-            'nick' => qr/^[[:alnum:]_][[:alnum:]_-]+$/,
+            'nick' => qr/^[[:alnum:]_][[:alnum:]_-]*$/,
             'tsrange' => qr/^[[(].*,.*[])]$/,
         },
         description => 'Get partial history of privilege level changes for employee with the given nick ' . 
@@ -1254,10 +1268,10 @@ EOH
         handler => {
             GET => 'handler_history_get', 
         },
-        acl_profile => 'admin',
-        cli => 'sched history nick $NICK $TSRANGE',
+        acl_profile => 'inactive',
+        cli => 'schedule history nick $NICK $TSRANGE',
         validations => {
-            'nick' => qr/^[[:alnum:]_][[:alnum:]_-]+$/,
+            'nick' => qr/^[[:alnum:]_][[:alnum:]_-]*$/,
             'tsrange' => qr/^[[(].*,.*[])]$/,
         },
         description => 'Get partial history of schedule changes for employee with the given nick ' . 
@@ -1275,6 +1289,7 @@ EOH
         parent => 'priv/history',
         handler => {
             GET => 'handler_history_get_phid',
+            POST => 'handler_history_post_phid',
             DELETE => 'handler_history_delete_phid',
         },
         acl_profile => 'admin',
@@ -1282,11 +1297,11 @@ EOH
         validations => {
             'phid' => 'Int',
         },
-        description => 'Retrieves (GET) or deletes (DELETE) a single privilege history record by its PHID',
+        description => 'Retrieves (GET), updates (POST), or deletes (DELETE) a single privilege history record by its PHID',
         documentation => <<'EOH',
 =pod
 
-Retrieves (GET) or deletes (DELETE) a single privilege history record by its
+Retrieves (GET), updates (POST), or deletes (DELETE) a single privilege history record by its
 PHID.
 
 =over
@@ -1295,6 +1310,11 @@ PHID.
 
 Retrieves a privhistory record by its PHID.
 
+=item * POST
+
+Updates a privilege history record by its PHID. The 'phid' and 'eid'
+properties cannot be changed in this way.
+
 =item * DELETE
 
 Deletes a privhistory record by its PHID.
@@ -1311,19 +1331,20 @@ EOH
         parent => 'schedule/history',
         handler => {
             GET => 'handler_history_get_shid',
+            POST => 'handler_history_post_shid',
             DELETE => 'handler_history_delete_shid',
         },
         acl_profile => 'admin',
-        cli => 'sched history phid $PHID',
+        cli => 'schedule history phid $PHID',
         validations => {
             'shid' => 'Int',
         },
-        description => 'Retrieves (GET) or deletes (DELETE) a single schedule history record by its SHID',
+        description => 'Retrieves (GET), updates (POST), or deletes (DELETE) a single schedule history record by its SHID',
         documentation => <<'EOH',
 =pod
 
-Retrieves (GET) or deletes (DELETE) a single schedule history record by its
-SHID.
+Retrieves (GET), updates (POST), or deletes (DELETE) a single schedule
+history record by its SHID.
 
 =over
 
@@ -1331,6 +1352,11 @@ SHID.
 
 Retrieves a schedule history record by its SHID.
 
+=item * POST
+
+Updates a schedule history record by its SHID. The 'shid' and 'eid'
+properties cannot be changed in this way.
+
 =item * DELETE
 
 Deletes a schedule history record by its SHID.
@@ -1373,7 +1399,7 @@ EOH
             GET => 'handler_history_self', 
         },
         acl_profile => 'inactive',
-        cli => 'sched history self [$TSRANGE]',
+        cli => 'schedule history self [$TSRANGE]',
         validations => {
             'tsrange' => qr/^[[(].*,.*[])]$/,
         },
@@ -1540,7 +1566,7 @@ EOH
         acl_profile => 'admin', 
         cli => 'interval nick $NICK $TSRANGE',
         validations => {
-            'nick' => qr/^[[:alnum:]_][[:alnum:]_-]+$/,
+            'nick' => qr/^[[:alnum:]_][[:alnum:]_-]*$/,
             'tsrange' => qr/^[[(].*,.*[])]$/,
         },
         description => 'Retrieve an arbitrary employee\'s intervals over the given tsrange',
@@ -1763,7 +1789,7 @@ EOH
         acl_profile => 'admin', 
         cli => 'lock nick $NICK $TSRANGE',
         validations => {
-            'nick' => qr/^[[:alnum:]_][[:alnum:]_-]+$/,
+            'nick' => qr/^[[:alnum:]_][[:alnum:]_-]*$/,
             'tsrange' => qr/^[[(].*,.*[])]$/,
         },
         description => 'Retrieve an arbitrary employee\'s locks over the given tsrange',
@@ -1859,7 +1885,7 @@ EOH
         acl_profile => 'admin', 
         cli => 'priv nick $NICK [$TIMESTAMP]',
         validations => {
-            'nick' => qr/^[[:alnum:]_][[:alnum:]_-]+$/,
+            'nick' => qr/^[[:alnum:]_][[:alnum:]_-]*$/,
         },
         description => 'Get the present privlevel of arbitrary employee, or with optional timestamp, that employee\'s privlevel as of that timestamp',
         documentation => <<'EOH',
@@ -1924,7 +1950,7 @@ EOH
         handler => {
             GET => 'handler_schedule_all',
         },
-        acl_profile => 'admin', 
+        acl_profile => 'inactive', 
         cli => 'schedule all',
         description => 'Retrieves (GET) all non-disabled schedules',
         documentation => <<'EOH',
@@ -2007,6 +2033,8 @@ Sample JSON:
         "[2014-09-25 12:30, 2014-09-25 16:30)"
     ] }
 
+(Optionally, an scode can be assigned by including an "scode" property.)
+
 Read on for details:
 
 First, a set of scratch intervals is created in the 'schedintvls' table.
@@ -2045,7 +2073,7 @@ EOH
         acl_profile => 'admin', 
         cli => 'schedule nick $NICK [$TIMESTAMP]',
         validations => {
-            'nick' => qr/^[[:alnum:]_][[:alnum:]_-]+$/,
+            'nick' => qr/^[[:alnum:]_][[:alnum:]_-]*$/,
         },
         description => 'Get the current schedule of arbitrary employee, or with optional timestamp, that employee\'s schedule as of that timestamp',
         documentation => <<'EOH',
@@ -2058,6 +2086,61 @@ is present, the schedule as of that timestamp is retrieved.
 EOH
     },
 
+    # /schedule/scode/:scode
+    'schedule/scode/:scode' => 
+    { 
+        parent => 'schedule',
+        handler => {
+            GET => 'handler_get_schedule_scode',
+            PUT => 'handler_put_schedule_scode', 
+            DELETE => 'handler_delete_schedule_scode',
+        },
+        acl_profile => {
+            GET => 'inactive',
+            PUT => 'admin', 
+            DELETE => 'admin',
+        },
+        cli => 'schedule scode $scode',
+        validations => {
+            'scode' => qr/^[[:alnum:]_][[:alnum:]_-]*$/,
+        },
+        description => 'Retrieves, updates, or deletes a schedule by its scode',
+        documentation => <<'EOH',
+=pod
+
+This resource makes it possible to GET, PUT, or DELETE a schedule by its scode.
+
+=over
+
+=item * GET
+
+An integer scode must be given as an URI parameter. If a schedule
+with this scode is found, it is returned in the payload.
+
+=item * PUT
+
+This resource/method provides a way to set (modify) the 'scode', 'remark'
+and/or 'disabled' fields of a schedule record. Simply provide the property (or
+properties) and the new value(s) in the request body, e.g.:
+
+    { "scode" : "WIGWAM" }
+
+or
+
+    { "remark" : "foobar", "disabled" : "t" }
+
+Properties other than these three cannot be modified using this resource.
+
+=item * DELETE
+
+The scode must be given as an URI parameter. If found, the schedule with that
+scode will be deleted in an atomic operation. If the operation is sucessful the
+return status will be "OK".
+
+=back
+EOH
+    },
+
     # /schedule/self/?:ts
     'schedule/self/?:ts' => 
     { 
@@ -2087,7 +2170,11 @@ EOH
             PUT => 'handler_put_schedule_sid', 
             DELETE => 'handler_delete_schedule_sid',
         },
-        acl_profile => 'admin', 
+        acl_profile => {
+            GET => 'inactive',
+            PUT => 'admin',
+            DELETE => 'admin',
+        },
         cli => 'schedule sid $SID',
         validations => {
             'sid' => 'Int',
@@ -2107,13 +2194,17 @@ with this SID is found, it is returned in the payload.
 
 =item * PUT
 
-This resource/method provides a way to set (modify) the 'remark' and 'disabled'
-fields of a schedule record. Simply provide the properties and their new values
-in the request body, e.g.:
+This resource/method provides a way to set (modify) the 'scode', 'remark'
+and/or 'disabled' fields of a schedule record. Simply provide the property (or
+properties) and the new value(s) in the request body, e.g.:
+
+    { "scode" : "WIGWAM" }
+
+or
 
     { "remark" : "foobar", "disabled" : "t" }
 
-Properties other than these two cannot be modified using this resource.
+Properties other than these three cannot be modified using this resource.
 
 =item * DELETE
 
