@@ -67,11 +67,11 @@ App::Dochazka::REST::Dispatch::Shared - Shared dispatch functions
 
 =head1 VERSION
 
-Version 0.407
+Version 0.427
 
 =cut
 
-our $VERSION = '0.407';
+our $VERSION = '0.427';
 
 my $fail = $CELL->status_not_ok;
 
@@ -96,6 +96,7 @@ use Exporter qw( import );
 our @EXPORT_OK = qw( 
     shared_first_pass_lookup
     shared_entity_check
+    shared_get_employee
     shared_insert_employee
     shared_update_employee
     shared_update_schedule
@@ -105,6 +106,7 @@ our @EXPORT_OK = qw(
     shared_employee_acl_part1
     shared_employee_acl_part2
     shared_update_activity
+    shared_update_history
     shared_insert_activity
     shared_insert_interval
     shared_insert_lock
@@ -145,10 +147,10 @@ This should be used only for resources that require an exact match.
 =cut
 
 sub shared_first_pass_lookup {
-    my ( $self, $key, $value ) = @_;
+    my ( $d_obj, $key, $value ) = @_;
     $log->debug( "Entering " . __PACKAGE__ . "::shared_first_pass_lookup with key $key, value $value" );
 
-    my $conn = $self->context->{'dbix_conn'};
+    my $conn = $d_obj->context->{'dbix_conn'};
     my ( $status, $thing );
 
     if ( $key eq 'AID' ) {
@@ -163,6 +165,9 @@ sub shared_first_pass_lookup {
     } elsif ( $key eq 'nick' ) {
         $thing = 'employee';
         $status = App::Dochazka::REST::Model::Employee->load_by_nick( $conn, $value );
+    } elsif ( $key eq 'sec_id' ) {
+        $thing = 'employee';
+        $status = App::Dochazka::REST::Model::Employee->load_by_sec_id( $conn, $value );
     } elsif ( $key eq 'IID' ) {
         $thing = 'interval';
         $status = App::Dochazka::REST::Model::Interval->load_by_iid( $conn, $value );
@@ -178,19 +183,22 @@ sub shared_first_pass_lookup {
     } elsif ( $key eq 'SID' ) {
         $thing = 'schedule';
         $status = App::Dochazka::REST::Model::Schedule->load_by_sid( $conn, $value );
+    } elsif ( $key eq 'scode' ) {
+        $thing = 'schedule';
+        $status = App::Dochazka::REST::Model::Schedule->load_by_scode( $conn, $value );
     } else {
-        die "AAAAAGAAAAGDDGASASAGAGGGGGAAHH!";
+        die "shared_first_pass_lookup could not do anything with key $key!";
     }
 
     if ( $status->level eq 'NOTICE' and $status->code eq 'DISPATCH_NO_RECORDS_FOUND' ) {
-        $self->mrest_declare_status( code => 404,
+        $d_obj->mrest_declare_status( code => 404,
             explanation => 'DISPATCH_SEARCH_EMPTY',
             args => [ $thing, "$key equals $value" ],
         );
         return;
     }
     if ( $status->not_ok ) {
-        $self->mrest_declare_status( code => 500, explanation => $status->code,
+        $d_obj->mrest_declare_status( code => 500, explanation => $status->code,
             args => $status->args 
         );
         return;
@@ -206,26 +214,26 @@ Check request entity for presence of properties
 =cut
 
 sub shared_entity_check {
-    my ( $self, @props ) = @_;
+    my ( $d_obj, @props ) = @_;
     $log->debug( "Entering " . __PACKAGE__ . "::shared_entity_check with properties " . 
-        join( ' ', @props ) . " and entity: " . Dumper( $self->context->{'request_entity'} ) );
+        join( ' ', @props ) . " and entity: " . Dumper( $d_obj->context->{'request_entity'} ) );
 
-    my $entity = $self->context->{'request_entity'};
+    my $entity = $d_obj->context->{'request_entity'};
     if ( not $entity ) {
-        $self->mrest_declare_status( code => 400, 
+        $d_obj->mrest_declare_status( code => 400, 
             explanation => 'DISPATCH_ENTITY_MISSING'
         );
         return $fail;
     }
     if ( ref( $entity ) ne 'HASH' ) {
-        $self->mrest_declare_status( code => 400, 
+        $d_obj->mrest_declare_status( code => 400, 
             explanation => 'DISPATCH_ENTITY_NOT_KEY_VALUE'
         );
         return $fail;
     }
     foreach my $p ( @props ) {
         if ( not $entity->{$p} ) {
-            $self->mrest_declare_status( code => 400, 
+            $d_obj->mrest_declare_status( code => 400, 
                 explanation => 'DISPATCH_PROP_MISSING_IN_ENTITY', args => [ $p ] 
             );
             return $fail;
@@ -235,11 +243,44 @@ sub shared_entity_check {
 }
 
 
+=head2 shared_get_employee
+
+=cut
+
+sub shared_get_employee {
+    my ( $d_obj, $pass, $key, $value ) = @_;
+    $log->debug( "Entering " . __PACKAGE__ . "::shared_get_employee" ); 
+
+    # first pass
+    if ( $pass == 1 ) {
+        #
+        # ACL check - passerby can only look up him- or herself
+        #
+        if ( ! acl_check_is_me( $d_obj, ( lc $key ) => $value ) ) {
+            $d_obj->mrest_declare_status( code => 403, explanation => "DISPATCH_KEEP_TO_YOURSELF" );
+            return 0;
+        }
+        #
+        # 404 check
+        #
+        my $emp = shared_first_pass_lookup( $d_obj, $key, $value );
+        return 0 unless $emp;
+        $d_obj->context->{'stashed_employee_object'} = $emp;
+        return 1;
+    }
+
+    # second pass
+    return $CELL->status_ok( 'DISPATCH_EMPLOYEE_FOUND',
+        payload => $d_obj->context->{'stashed_employee_object'},
+    );
+}
+
+
 =head2 shared_update_employee
 
 Takes three arguments:
 
-    - $self is the App::Dochazka::REST::Dispatch object
+    - $d_obj is the App::Dochazka::REST::Dispatch object
     - $emp is an employee object (blessed hashref)
     - $over is a hashref with zero or more employee properties and new values
 
@@ -248,7 +289,7 @@ The values from $over replace those in $emp
 =cut
 
 sub shared_update_employee {
-    my ( $self, $emp, $over ) = @_;
+    my ( $d_obj, $emp, $over ) = @_;
     $log->debug("Entering " . __PACKAGE__ . "::shared_update_employee" );
     $log->debug("Updating employee: " . Dumper( $emp ) );
     $log->debug("With key:value pairs " . Dumper( $over ) );
@@ -256,12 +297,12 @@ sub shared_update_employee {
     ACL: {
         my $explanation = "Update operations require at least one key:value pair in the request entity";
         if ( ref( $over ) ne 'HASH' ) {
-            $self->mrest_declare_status( code => 400, explanation => $explanation );
+            $d_obj->mrest_declare_status( code => 400, explanation => $explanation );
             return $fail;
         }
         delete $over->{'eid'};
         if ( $over == {} ) {
-            $self->mrest_declare_status( code => 400, explanation => $explanation );
+            $d_obj->mrest_declare_status( code => 400, explanation => $explanation );
             return $fail;
         } 
     }
@@ -272,8 +313,8 @@ sub shared_update_employee {
     # Authen::Passphrase
     hash_the_password( $over );
 
-    return $emp->update( $self->context ) if pre_update_comparison( $emp, $over );
-    $self->mrest_declare_status( code => 400, explanation => "DISPATCH_ILLEGAL_ENTITY" );
+    return $emp->update( $d_obj->context ) if pre_update_comparison( $emp, $over );
+    $d_obj->mrest_declare_status( code => 400, explanation => "DISPATCH_ILLEGAL_ENTITY" );
     return $fail;
 }
 
@@ -282,7 +323,7 @@ sub shared_update_employee {
 
 Called from handlers in L<App::Dochazka::REST::Dispatch>. Takes three arguments:
 
-    - $self is the App::Dochazka::REST::Dispatch object
+    - $d_obj is the App::Dochazka::REST::Dispatch object
     - $ignore_me will be undef
     - $new_emp_props is a hashref with employee properties and their values (guaranteed to contain 'nick')
 
@@ -290,7 +331,7 @@ Called from handlers in L<App::Dochazka::REST::Dispatch>. Takes three arguments:
 
 sub shared_insert_employee {
     $log->debug( "Entered " . __PACKAGE__ . "::shared_insert_employee" );
-    my ( $self, $ignore_me, $new_emp_props ) = validate_pos( @_,
+    my ( $d_obj, $ignore_me, $new_emp_props ) = validate_pos( @_,
         { isa => 'App::Dochazka::REST::Dispatch' },
         { type => UNDEF },
         { type => HASHREF },
@@ -307,7 +348,7 @@ sub shared_insert_employee {
     my $emp = App::Dochazka::REST::Model::Employee->spawn( @filtered_args );
 
     # execute the INSERT db operation
-    return $emp->insert( $self->context );
+    return $emp->insert( $d_obj->context );
 }
 
 
@@ -315,7 +356,7 @@ sub shared_insert_employee {
 
 Takes three arguments:
 
-    - $self is the dispatch (App::Dochazka::REST::Dispatch) object
+    - $d_obj is the dispatch (App::Dochazka::REST::Dispatch) object
     - $sched is a schedule object (blessed hashref)
     - $over is a hashref with zero or more schedule properties and new values
 
@@ -324,7 +365,7 @@ The values from C<$over> replace those in C<$emp>.
 =cut
 
 sub shared_update_schedule {
-    my ( $self, $sched, $over ) = validate_pos( @_,
+    my ( $d_obj, $sched, $over ) = validate_pos( @_,
         { isa => 'App::Dochazka::REST::Dispatch' },
         { isa => 'App::Dochazka::REST::Model::Schedule' },
         { type => HASHREF },
@@ -335,10 +376,10 @@ sub shared_update_schedule {
     delete $over->{'schedule'} if exists $over->{'schedule'};
     if ( pre_update_comparison( $sched, $over ) ) {
         $log->debug( "After pre_update_comparison: " . Dumper $sched );
-        return $sched->update( $self->context );
+        return $sched->update( $d_obj->context );
     }
 
-    $self->mrest_declare_status( 
+    $d_obj->mrest_declare_status( 
         code => 400, 
         explanation => "Cannot update schedule due to invalid input",
     );
@@ -410,7 +451,7 @@ Shared GET handler for 'priv' and 'schedule' lookups. Takes four arguments:
 
 =over
 
-=item C<$self> - dispatch object
+=item C<$d_obj> - dispatch object
 
 =item C<$t> - either 'priv' or 'schedule'
 
@@ -425,7 +466,7 @@ Shared GET handler for 'priv' and 'schedule' lookups. Takes four arguments:
 =cut
 
 sub shared_get_privsched {
-    my ( $self, $t, $pass, $key, $value ) = @_;
+    my ( $d_obj, $t, $pass, $key, $value ) = @_;
     $log->debug( "Entering " . __PACKAGE__ . ":shared_get_privsched" ); 
 
     # first pass
@@ -433,16 +474,16 @@ sub shared_get_privsched {
         #
         # 403 (ACL) check - passerby can only look up him- or herself
         #
-        if ( ! acl_check_is_me( $self, ( lc $key ) => $value ) ) {
-            $self->mrest_declare_status( code => 403, explanation => "DISPATCH_KEEP_TO_YOURSELF" );
+        if ( ! acl_check_is_me( $d_obj, ( lc $key ) => $value ) ) {
+            $d_obj->mrest_declare_status( code => 403, explanation => "DISPATCH_KEEP_TO_YOURSELF" );
             return 0;
         }
         # 
         # 404 check
         #
-        my $emp = shared_first_pass_lookup( $self, $key => $value );
+        my $emp = shared_first_pass_lookup( $d_obj, $key => $value );
         return 0 unless $emp;
-        $self->context->{'stashed_employee_object'} = $emp;
+        $d_obj->context->{'stashed_employee_object'} = $emp;
         return 1;
     }
     
@@ -454,18 +495,32 @@ sub shared_get_privsched {
         'priv' => \&priv_by_eid,
         'schedule' => \&schedule_by_eid,
     );
-    my $emp = $self->context->{'stashed_employee_object'};
+    my $emp = $d_obj->context->{'stashed_employee_object'};
     my $eid = $emp->eid;
-    my $ts = $self->context->{'mapping'}->{'ts'};
-    my $conn = $self->context->{'dbix_conn'};
+    my $ts = $d_obj->context->{'mapping'}->{'ts'};
+    my $conn = $d_obj->context->{'dbix_conn'};
 
     # - run priv_by_eid or schedule_by_eid, as appropriate
     my $return_value = $dispatch{$t}->( $conn, $eid, $ts );
 
-    # on success, $return_value will be a SCALAR like 'inactive' (priv) or a long JSON string (schedule)
+    # on success, $return_value will be a SCALAR like 'inactive' (priv) or 8 (SID of schedule)
     if ( ref( $return_value ) ne 'App::CELL::Status' ) {
+
+        if ( $return_value and $t eq 'schedule' ) {
+            # $return_value is SID of the schedule, but we want the schedule itself
+            my $status = App::Dochazka::REST::Model::Schedule->load_by_sid( $conn, $return_value );
+            $return_value = $status->payload;
+        }
+
         my @privsched = ( $t, $return_value );
         if ( $ts ) {
+            if ( ! $return_value ) {
+                $d_obj->mrest_declare_status( 
+                    code => 404, 
+                    explanation => "EID $eid has no $t assigned as of $ts" 
+                );
+                return $CELL->status_not_ok;
+            }
             return $CELL->status_ok(
                 'DISPATCH_EMPLOYEE_' . uc( $t ) . '_AS_AT',
                 args => [ $ts, $emp->nick, $return_value ],
@@ -477,6 +532,13 @@ sub shared_get_privsched {
                 },
             );
         } else {
+            if ( ! $return_value ) {
+                $d_obj->mrest_declare_status( 
+                    code => 404, 
+                    explanation => "EID $eid has no $t assigned"
+                );
+                return $CELL->status_not_ok;
+            }
             return $CELL->status_ok(
                 'DISPATCH_EMPLOYEE_' . uc( $t ),
                 args => [ $emp->nick, $return_value ],
@@ -502,16 +564,16 @@ EID. Returns boolean 1 or 0, where 1 means "ACL check passed".
 =cut
 
 sub shared_employee_acl_part1 {
-    my ( $self, $this_emp ) = @_;
+    my ( $d_obj, $this_emp ) = @_;
     $log->debug( "Entering " . __PACKAGE__ . "::shared_employee_acl_part1" );
 
-    my $context = $self->context;
+    my $context = $d_obj->context;
     my $cp = $context->{'current_priv'} || "none";
 
     # insert
     if ( ! defined( $this_emp ) ) {
         if ( $cp ne 'admin' ) {
-            $self->mrest_declare_status( code => 403,
+            $d_obj->mrest_declare_status( code => 403,
                 explanation => "Only administrators can insert new employee records"
             );
             return 0;
@@ -526,7 +588,7 @@ sub shared_employee_acl_part1 {
             return 1;
         }
     }
-    $self->mrest_declare_status( code => 403, explanation => "DISPATCH_KEEP_TO_YOURSELF" );
+    $d_obj->mrest_declare_status( code => 403, explanation => "DISPATCH_KEEP_TO_YOURSELF" );
     return 0;
 }
 
@@ -542,10 +604,10 @@ Returns boolean 1 or 0, where 1 means "ACL check passed".
 =cut
 
 sub shared_employee_acl_part2 {
-    my ( $self ) = @_;
+    my ( $d_obj ) = @_;
     $log->debug( "Entering " . __PACKAGE__ . "::shared_employee_acl_part2" );
 
-    my $context = $self->context;
+    my $context = $d_obj->context;
     my $cp = $context->{'current_priv'} || 'none';
 
     if ( $cp eq 'admin' ) {
@@ -556,15 +618,24 @@ sub shared_employee_acl_part2 {
         map { $lut{$_} = ''; } @{ $site->DOCHAZKA_PROFILE_EDITABLE_FIELDS->{$cp} };
         foreach my $prop ( keys %{ $context->{'request_entity'} } ) {
             next if exists $lut{$prop};
-            $self->mrest_declare_status( code => 403,
-                explanation => "Employees with privlevel $cp are not allowed to update $prop property"
+            $d_obj->mrest_declare_status(
+                $CELL->status_err( 
+                    'DOCHAZKA_ACL_VIOLATION', 
+                    args => [ $cp, "update $prop property" ],
+                    http_code => 403,
+                    uri_path => $context->{'uri_path'},
+                )
             );
             return 0;
         }
         return 1;
     }
-    $self->mrest_declare_status( code => 403,
-        explanation => "Employees with privlevel $cp are not allowed to update employee profiles"
+    $d_obj->mrest_declare_status(
+        $CELL->status_err( 
+            'DOCHAZKA_ACL_VIOLATION', 
+            args => [ $cp, "update employee profiles" ],
+            http_code => 403,
+        )
     );
     return 0;
 }
@@ -574,7 +645,7 @@ sub shared_employee_acl_part2 {
 
 Takes three arguments:
 
-  - $self is the dispatch object
+  - $d_obj is the dispatch object
   - $act is an activity object (blessed hashref)
   - $over is a hashref with zero or more activity properties and new values
 
@@ -583,11 +654,33 @@ The values from $over replace those in $act
 =cut
 
 sub shared_update_activity {
-    my ( $self, $act, $over ) = @_;
+    my ( $d_obj, $act, $over ) = @_;
     $log->debug("Entering " . __PACKAGE__ . "::shared_update_activity" );
     delete $over->{'aid'} if exists $over->{'aid'};
-    return $act->update( $self->context ) if pre_update_comparison( $act, $over );
-    $self->mrest_declare_status( code => 400, explanation => "DISPATCH_ILLEGAL_ENTITY" );
+    return $act->update( $d_obj->context ) if pre_update_comparison( $act, $over );
+    $d_obj->mrest_declare_status( code => 400, explanation => "DISPATCH_ILLEGAL_ENTITY" );
+    return $fail;
+}
+
+
+=head2 shared_update_history
+
+Takes three arguments:
+
+  - $d_obj is the dispatch object
+  - $obj is a (priv/schedule) history object (blessed hashref)
+  - $over is a hashref with zero or more history properties and new values
+
+The values from $over replace those in $obj
+
+=cut
+
+sub shared_update_history {
+    my ( $d_obj, $obj, $over ) = @_;
+    $log->debug("Entering " . __PACKAGE__ . "::shared_update_history" );
+    delete $over->{'eid'} if exists $over->{'eid'};
+    return $obj->update( $d_obj->context ) if pre_update_comparison( $obj, $over );
+    $d_obj->mrest_declare_status( code => 400, explanation => "DISPATCH_ILLEGAL_ENTITY" );
     return $fail;
 }
 
@@ -600,7 +693,7 @@ to be an activity object to be inserted.
 =cut
 
 sub shared_insert_activity {
-    my ( $self, $code, $props ) = validate_pos( @_,
+    my ( $d_obj, $code, $props ) = validate_pos( @_,
         { isa => 'App::Dochazka::REST::Dispatch' },
         { type => SCALAR },
         { type => HASHREF },
@@ -618,7 +711,7 @@ sub shared_insert_activity {
     my $act = App::Dochazka::REST::Model::Activity->spawn( @filtered_args );
 
     # execute the INSERT db operation
-    return $act->insert( $self->context );
+    return $act->insert( $d_obj->context );
 }
 
 
@@ -629,10 +722,10 @@ Shared routine for inserting attendance intervals.
 =cut
 
 sub shared_insert_interval {
-    my ( $self ) = @_;
+    my ( $d_obj ) = @_;
     $log->debug("Reached " . __PACKAGE__ . "::shared_insert_interval" );
 
-    return shared_insert_intlock( $self, 'Interval' );
+    return shared_insert_intlock( $d_obj, 'Interval' );
 }
 
 
@@ -643,10 +736,10 @@ Shared routine for inserting lock intervals.
 =cut
 
 sub shared_insert_lock {
-    my ( $self ) = @_;
+    my ( $d_obj ) = @_;
     $log->debug("Reached " . __PACKAGE__ . "::shared_insert_lock" );
 
-    return shared_insert_intlock( $self, 'Lock' );
+    return shared_insert_intlock( $d_obj, 'Lock' );
 }
 
 
@@ -655,10 +748,10 @@ sub shared_insert_lock {
 =cut
 
 sub shared_insert_intlock {
-    my ( $self, $intlock ) = @_;
+    my ( $d_obj, $intlock ) = @_;
     $log->debug("Reached " . __PACKAGE__ . "::shared_insert_intlock with $intlock" );
 
-    my $context = $self->context;
+    my $context = $d_obj->context;
 
     my %proplist_before = %{ $context->{'request_entity'} };
     $log->debug( "Properties before filter: " . join( ' ', keys %proplist_before ) );
@@ -692,7 +785,7 @@ sub shared_insert_intlock {
 
 Takes three arguments:
 
-  - $self is the dispatch object
+  - $d_obj is the dispatch object
   - $int is an interval or lock object (blessed hashref)
   - $over is a hashref with zero or more interval properties and new values
 
@@ -701,10 +794,10 @@ The values from $over replace those in $int
 =cut
 
 sub shared_update_intlock {
-    my ( $self, $int, $over ) = @_;
+    my ( $d_obj, $int, $over ) = @_;
     $log->debug("Entering " . __PACKAGE__ . "::shared_update_intlock" );
 
-    my $context = $self->context;
+    my $context = $d_obj->context;
 
     # determine whether we have been passed an interval or lock and set $idv accordingly
     my $class = ref( $int );
@@ -724,7 +817,7 @@ sub shared_update_intlock {
     # $over into $int
     return $int->update( $context ) if pre_update_comparison( $int, $over );
     $log->notice( "Failed pre_update_comparison" );
-    $self->mrest_declare_status( code => 400, explanation => 'Check request entity syntax' );
+    $d_obj->mrest_declare_status( code => 400, explanation => 'Check request entity syntax' );
     return $fail;
 }
 
