@@ -67,11 +67,11 @@ App::Dochazka::REST::Test - Test helper functions
 
 =head1 VERSION
 
-Version 0.407
+Version 0.427
 
 =cut
 
-our $VERSION = '0.407';
+our $VERSION = '0.427';
 
 
 
@@ -366,9 +366,10 @@ Create testing employee with 'active' privilege
 
 sub create_active_employee {
     my ( $test ) = @_;
+    note('create active employee');
     my $eid_of_active = create_testing_employee( { nick => 'active', password => 'active' } )->{'eid'};
     my $status = req( $test, 201, 'root', 'POST', "priv/history/eid/$eid_of_active", 
-        '{ "effective":"1000-01-01", "priv":"active" }' );
+        '{ "effective":"1892-01-01", "priv":"active" }' );
     ok( $status->ok, "Create active employee 2" );
     is( $status->code, 'DOCHAZKA_CUD_OK', "Create active employee 3" );
     return $eid_of_active;
@@ -383,12 +384,13 @@ Create testing employee with 'active' privilege
 
 sub create_inactive_employee {
     my ( $test ) = @_;
+    note( 'create inactive employee' );
     my $eid_of_inactive = create_testing_employee( { nick => 'inactive', password => 'inactive' } )->{'eid'};
     my $status = req( $test, 201, 'root', 'POST', "priv/history/eid/$eid_of_inactive", 
-        '{ "effective":"1000-01-01", "priv":"inactive" }' );
+        '{ "effective":"1892-01-01", "priv":"inactive" }' );
     ok( $status->ok, "Create inactive employee 2" );
     is( $status->code, 'DOCHAZKA_CUD_OK', "Create inactive employee 3" );
-    return $eid_of_inactive;
+    return $eid_of_inactive; 
 }
 
 
@@ -400,6 +402,7 @@ Tests will need to set up and tear down testing employees (takes EID)
 
 sub delete_testing_employee {
     my $eid = shift;  
+    note( "delete testing employee with EID $eid" );
     my $status = App::Dochazka::REST::Model::Employee->load_by_eid( $dbix_conn, $eid );
     if ( $status->not_ok ) {
         diag( Dumper $status );
@@ -519,6 +522,8 @@ object as its only argument.
 sub create_testing_schedule {
     my ( $test ) = @_;
 
+    note( "Create a testing schedule" );
+
     my $intvls = { "schedule" => [
         "[2000-01-02 12:30, 2000-01-02 16:30)",
         "[2000-01-02 08:00, 2000-01-02 12:00)",
@@ -526,7 +531,7 @@ sub create_testing_schedule {
         "[2000-01-01 08:00, 2000-01-01 12:00)",
         "[1999-12-31 12:30, 1999-12-31 16:30)",
         "[1999-12-31 08:00, 1999-12-31 12:00)",
-    ] };
+    ], "scode" => 'KOBOLD' };
     my $intvls_json = JSON->new->utf8->canonical(1)->encode( $intvls );
     #
     # - request as root 
@@ -535,6 +540,7 @@ sub create_testing_schedule {
     is( $status->code, 'DISPATCH_SCHEDULE_INSERT_OK' );
     ok( exists $status->{'payload'} );
     ok( exists $status->payload->{'sid'} );
+    ok( exists $status->payload->{'scode'} );
 
     return $status->payload->{'sid'};
 }
@@ -744,6 +750,7 @@ sub do_select_single {
     #diag( "do_select_single: SQL statement is $sql" ) if $sql;
     #diag( "do_select_single: keys are ", join(', ', @keys) ) if @keys;
     my $status = select_single( conn => $conn, sql => $sql, keys => \@keys );
+    #diag( Dumper $status );
     is( $status->level, 'OK' );
     is( $status->code, 'DISPATCH_RECORDS_FOUND' );
     ok( $status->payload );
